"""
The coalib package is a collection of various subpackages regarding
writing, executing and editing bears. Various other packages such as
formatting and settings are also included in coalib.
"""


import sys
from os.path import join, dirname


VERSION_FILE = join(dirname(__file__), 'VERSION')


def get_version():
    with open(VERSION_FILE, 'r') as ver:
        return ver.readline().strip()


VERSION = get_version()
__version__ = VERSION


def assert_supported_version():
    if sys.version_info < (3, 4):
        print('coala supports only python 3.4 or later.')
        exit(4)



from collections import defaultdict

from coalib.bearlib.aspects.collections import AspectList


class bearclass(type):
    """
    Metaclass for :class:`coalib.bears.Bear.Bear` and therefore all bear
    classes.

    Pushing bears into the future... ;)
    """

    # by default a bear class has no aspects
    aspects = defaultdict(lambda: AspectList([]))

    def __new__(mcs, clsname, bases, clsattrs, *varargs, aspects=None):
        return type.__new__(mcs, clsname, bases, clsattrs, *varargs)

    def __init__(cls, clsname, bases, clsattrs, *varargs, aspects=None):
        """
        Initializes the ``.aspects`` dict on new bear classes from the mapping
        given to the keyword-only `aspects` argument.
        """
        type.__init__(cls, clsname, bases, clsattrs, *varargs)
        if aspects is not None:
            cls.aspects = defaultdict(
                lambda: AspectList([]),
                ((k, AspectList(v)) for (k, v) in dict(aspects).items()))
